/*  Copyright (C) 2008 Joshua Judson Rosen <rozzin@geekspace.com>.

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    version 3 as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING. If not, see
    <http://www.gnu.org/licenses/> or write to:

        The Free Software Foundation, inc.
        51 Franklin Street, Fifth Floor
        Boston, MA 02110-1301
        USA
*/

#include <stdlib.h>
#include "constants.h"
#include "random.h"

#include <math.h>
#include <cairo.h>

#include "graphics.h"
#include "figure.h"
#include "radial.h"
#include "line.h"

#include <glib.h>

static void
exec_figure (struct figure_generator *figure, cairo_t *cr)
{
  double x, y;
  int i;

  cairo_get_current_point (cr, &x, &y);
  cairo_new_sub_path (cr);

  cairo_move_to (cr, 0, 0);

  cairo_save (cr);
  for (i = 0; i < 4; i++) {
    cairo_save (cr);
    if (i) {
      if (i < 3) {
        cairo_scale (cr, 1, -1);
      }
      if (i > 1) {
        cairo_scale (cr, -1, 1);
      }
    }
    cairo_rotate (cr, M_PI_4);
/*     if (i > 1) { */
/*       cairo_scale (cr, -1, 1); */
/*     } */
/*     cairo_rotate (cr, M_PI_2 * (i % 2) + M_PI_4); */
    cairo_translate (cr, 0, figure->leg_scale/2);
    cairo_scale (cr, figure->leg_scale, figure->leg_scale);
    figure->leg->produce (figure->leg, cr);
    cairo_restore (cr);
  }
  cairo_restore (cr);

  /* head */
  cairo_save (cr);
  cairo_translate (cr, 0, figure->leg_scale/2 + figure->head_scale/2); /* .5); */
  cairo_scale (cr, figure->head_scale, figure->head_scale);
  figure->head->produce (figure->head, cr);
  cairo_restore (cr);

  if (figure->tail) {
    cairo_save (cr);
    cairo_translate (cr, 0, -figure->tail_scale /* -.3 */);
    cairo_scale (cr, figure->tail_scale, figure->tail_scale);
    figure->tail->produce (figure->tail, cr);
    cairo_restore (cr);
  }

  cairo_new_sub_path (cr);
  cairo_move_to (cr, x, y);
}

struct generator *
gen_figure (int level)
{
  struct figure_generator *g = g_new (struct figure_generator, 1);

  ((struct generator *) g)->produce = (producer_fn *) exec_figure;

  g->head = gen_radial (1 + level);
  g->head_scale = rndin (0.1, 0.2);

  g->leg = gen_line (1 + level);
  g->leg_scale = rndin (0.3, 0.6);

  if (rndprob (.8)) {
    g->tail = gen_line (1 + level);
    g->tail_scale = rndin (0.1, 0.4);
  } else {
    g->tail = NULL;
  }

  return (struct generator *) g;
}
