/*  Copyright (C) 2008 Joshua Judson Rosen <rozzin@geekspace.com>.

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    version 3 as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING. If not, see
    <http://www.gnu.org/licenses/> or write to:

        The Free Software Foundation, inc.
        51 Franklin Street, Fifth Floor
        Boston, MA 02110-1301
        USA
*/

#include <stdlib.h>
#include "constants.h"
#include "random.h"

#include <math.h>
#include <cairo.h>

#include "graphics.h"
#include "line.h"

#include <glib.h>

static void
exec_line (struct line_generator *line, cairo_t *cr)
{
  double x, y;
  int i;

  cairo_get_current_point (cr, &x, &y);
  cairo_new_sub_path (cr);

  cairo_move_to (cr, 0, line->len/2);
  if (line->ec1) {
    cairo_save (cr);
    cairo_translate (cr, 0, line->len/2);
    cairo_scale (cr, line->c1_scale, line->c1_scale);
    line->ec1->produce (line->ec1, cr);
    cairo_restore (cr);
  }

  cairo_line_to (cr, 0, -line->len/2);
  if (line->ec2) {
    cairo_save (cr);
    cairo_translate (cr, 0, -line->len/2);
    cairo_scale (cr, line->c2_scale, line->c2_scale);
    line->ec2->produce (line->ec2, cr);
    cairo_restore (cr);
  }

  if (line->c3) {
    for (i = 0;
         i < line->n;
         i++) {
      double pos = ((double) i) - ((double) line->n - 1) / 2;

      cairo_save (cr);
      cairo_translate (cr, 0,
                       line->len/2
                       * pos
                       / ((double) (line->n - 1) /* #spaces between children */
                          / 2)); /* use # children between [0,1], not [-1,1] */
      cairo_scale (cr, line->c3_scale, line->c3_scale);
      if (line->c3_taper) {
        double taper = 1 + (.5 * i/line->n);
        if (!line->c3_increase) {
          taper = 1/taper;
        }
        cairo_scale (cr, taper, taper);
      }
      if (line->c3_doublesided) {
        cairo_translate (cr, 1, 0);
        cairo_save (cr);
        cairo_rotate (cr, line->c3_angle - M_PI_2);
        line->c3->produce (line->c3, cr);
        cairo_restore (cr);
        cairo_scale (cr, -1, 1);
        cairo_translate (cr, 2, 0);
      }
      cairo_rotate (cr, line->c3_angle - M_PI_2);
      line->c3->produce (line->c3, cr);
      cairo_restore (cr);
    }
  }

  cairo_new_sub_path (cr);
  cairo_move_to (cr, x, y);
}

struct generator *
gen_line (int level)
{
  struct line_generator *g = g_new (struct line_generator, 1);

  ((struct generator *) g)->produce = (producer_fn *) exec_line;

  g->len = rndin (0.5, 1);
  g->n = rndin_lowbias (3, 10, 1.5);

  if (level < recursion_max && rndprob (0.5)) {
    g->c3 = generate_child (1 + level);
    g->c3_scale = rndin_lowbias (0.05, 0.7, 1.5);
    g->c3_doublesided = rndprob (0.8);
    g->c3_angle = rndin (-M_PI_4, M_PI_4);
    g->c3_taper = rndprob (0.3);
    g->c3_increase = rndprob (0.3);
  } else {
    g->c3 = NULL;
  }

  if (level < recursion_max && rndprob (0.75)) {
    g->ec2 = generate_child (1 + level);
    g->c2_scale = rndin (0.15, 0.6);
  } else {
    g->ec2 = NULL;
  }

  if (level < recursion_max && (!(g->ec2 || g->c3) || rndprob (0.75))) {
    g->ec1 = generate_child (1 + level);
    g->c1_scale = rndin (0.15, 0.6);
  } else {
    g->ec1 = NULL;
  }

  return (struct generator *) g;
}
