/*  Copyright (C) 2008 Joshua Judson Rosen <rozzin@geekspace.com>.

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    version 3 as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING. If not, see
    <http://www.gnu.org/licenses/> or write to:

        The Free Software Foundation, inc.
        51 Franklin Street, Fifth Floor
        Boston, MA 02110-1301
        USA
*/

#include <stdlib.h>
#include "constants.h"
#include "random.h"

#include <math.h>
#include <cairo.h>

#include "graphics.h"
#include "radial.h"

#include <glib.h>

#include <stdio.h>

void
exec_radial (struct radial_generator *radial, cairo_t *cr)
{
  double x, y;
  int i;

  cairo_get_current_point (cr, &x, &y);
  cairo_new_sub_path (cr);

  cairo_save (cr);

  for (i = -1;
       i < radial->n;
       i++)
    {
      double angle = M_PI*2 * i/radial->n; 

      double vx = -sin (angle) * .6, vy = cos (angle) * .6;

      cairo_line_to (cr, vx, vy);

      if (i >= 0 && radial->vertex_child) {
        if (angle > radial->angle_cutoff &&
            (M_PI*2 - angle) > radial->angle_cutoff)
          {
            continue;
          }

        cairo_save (cr);

        cairo_translate (cr, vx, vy);
        cairo_rotate (cr, angle);
        cairo_scale (cr, radial->vc_scale, radial->vc_scale);

        radial->vertex_child->produce (radial->vertex_child, cr);

        cairo_restore (cr);
      }
    }

  cairo_close_path (cr);

  if (radial->eye_child) {
    cairo_new_sub_path (cr);
    cairo_move_to (cr, 0, 0);

    cairo_save (cr);
    cairo_translate (cr, -(radial->eye_sep)/2, radial->eye_org);
    cairo_scale (cr, radial->ec_scale, radial->ec_scale);
    radial->eye_child->produce (radial->eye_child, cr);
    cairo_restore (cr);

    cairo_save (cr);
    cairo_translate (cr, radial->eye_sep/2, radial->eye_org);
    cairo_scale (cr, radial->ec_scale, radial->ec_scale);
    radial->eye_child->produce (radial->eye_child, cr);
    cairo_restore (cr);
  }

  if (radial->mouth_child) {
    cairo_new_sub_path (cr);
    cairo_move_to (cr, 0, 0);

    cairo_save (cr);
    cairo_translate (cr, 0, radial->mouth_org);
    cairo_scale (cr, radial->mc_scale, radial->mc_scale);
    radial->mouth_child->produce (radial->mouth_child, cr);
    cairo_restore (cr);
  }

  cairo_move_to (cr, x, y);

  cairo_restore (cr);
}

struct generator *
gen_radial (int recursion_level)
{
  struct radial_generator *rg = g_new (struct radial_generator, 1);

  ((struct generator *) rg)->produce = (producer_fn *) exec_radial;

  rg->n = (int) rndin_lowbias (3, 12, 2);

  if (rndprob (0.5)) {
    rg->vertex_child = generate_child (1 + recursion_level);
    rg->vc_scale = rndin (0.15, 0.7);
  }
  else {
    rg->vertex_child = NULL;
  }

  if (rndprob (0.6)) {
    rg->mouth_child = generate_child (1 + recursion_level);
    rg->mc_scale = rndin (0.1, 0.3);
  } else {
    rg->mouth_child = NULL;
  }

  if (rndprob (0.3)) {
    rg->eye_child = generate_child (1 + recursion_level);
    rg->ec_scale = rndin (0.1, 0.4);
    rg->eye_sep = rndin (0.2, 0.3);
    rg->eye_org = rndin (0.2, 0.3);
    rg->mouth_org = rndin (-0.2, -0.3);
  } else {
    rg->eye_child = NULL;
    rg->mouth_org = 0.0;
  }

  if (rndprob (0.5)) {
    rg->angle_cutoff = rndin (M_PI/2, M_PI/1.2);
  } else {
    rg->angle_cutoff = M_PI;
  }

  return (struct generator *) rg;
}
