/*  Copyright (C) 2008 Joshua Judson Rosen <rozzin@geekspace.com>.

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    version 3 as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING. If not, see
    <http://www.gnu.org/licenses/> or write to:

        The Free Software Foundation, inc.
        51 Franklin Street, Fifth Floor
        Boston, MA 02110-1301
        USA
*/

#include <math.h>
#include <stdlib.h>
#include "random.h"

double
rnd (void)
{
  return (double) rand () / (double) RAND_MAX;
}

double
rndin (double low, double high)
{
  double delta = high - low;
  return low + (delta * rnd ());
}

double
rndin_lowbias (double low, double high, double rnd_power)
{
  double delta = high - low;
  double r = rnd ();
  double res = low + (delta * pow(r, rnd_power));

  return res;
}

int
rndprob (double probability)
{
  return rnd () >= probability;
}

double
rndin_level (double low, double high, int level)
{ /* level starts at 0, but we'll shift the scale
     so that it starts at 1, here */
  return low + rnd () * (high - low) / ((double) level + 1);
}

int
rndprob_level (double probability, int level)
{
  return rndprob (probability * (1 - fmin ((double) level, 5) / 6));
}

double /* XXX--the format of the arguments seems silly... */
rational (int low, int high, int denominator)
{
  return round (rndin (low, high)) / denominator;
}

double
rnd11 (void)
{
  return rndin (-1.0, 1.0);
}
