/*  Copyright (C) 2008 Joshua Judson Rosen <rozzin@geekspace.com>.

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    version 3 as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING. If not, see
    <http://www.gnu.org/licenses/> or write to:

        The Free Software Foundation, inc.
        51 Franklin Street, Fifth Floor
        Boston, MA 02110-1301
        USA
*/

#include <stdlib.h>
#include "constants.h"
#include "random.h"

#include <math.h>
#include <cairo.h>

#include "graphics.h"
#include "spiral.h"

#include <glib.h>

void
exec_spiral (struct spiral_generator *spiral, cairo_t *cr)
{
  double x, y;
  int i;

  cairo_get_current_point (cr, &x, &y);
  cairo_new_sub_path (cr);

  if (spiral->centre_child) {
    cairo_save (cr);
    cairo_move_to (cr, 0, 0);
    cairo_scale (cr, spiral->centre_scale, spiral->centre_scale);
    spiral->centre_child->produce (spiral->centre_child, cr);
    cairo_restore (cr);
    cairo_new_sub_path (cr);
  }

  for (i = 1;
       i < spiral->len;
       i++) {
    double theta = 2 * M_PI * ((double) i) / ((double) spiral->ppr);
    double r = 0.15 * sqrt(theta);
    double vx = r * -sin(theta);
    double vy = r * cos(theta);

    cairo_line_to (cr, vx, vy);

    if (spiral->outer_child) {
      cairo_save (cr);
      cairo_translate (cr, vx, vy);
      cairo_scale (cr, spiral->outer_scale, spiral->outer_scale);
      cairo_rotate (cr, theta);
      spiral->outer_child->produce (spiral->outer_child, cr);
      cairo_restore (cr);
    }
  }

  cairo_new_sub_path (cr);
  cairo_move_to (cr, x, y);
}

struct generator *
gen_spiral (int recursion_level)
{
  struct spiral_generator *g = g_new (struct spiral_generator, 1);

  ((struct generator *) g)->produce = (producer_fn *) exec_spiral;

  switch (recursion_level) {
  case 0:
    g->len = rndin (25, 60); break;
  case 1:
    g->len = rndin (10, 30); break;
  case 2:
    g->len = rndin (5, 10); break;
  default: /* > 2 */
    g->len = 5; break;
  }

  g->aligned = rndprob (.5);

  if (g->aligned) {
    g->ppr = rndin_lowbias (3, 35, 2);
  } else {
    g->ppr = rational (3, 35, round (rndin (4, 7)));
  }

  if (rndprob (0.5)) {
    g->centre_child = generate_child (1 + recursion_level);
    g->centre_scale = rndin (0.15, 0.7);
  }
  else {
    g->centre_child = NULL;
  }

  if (rndprob (0.5)) {
    g->outer_child = generate_child (1 + recursion_level);
    g->outer_scale = rndin (0.05, 0.6);
  }
  else {
    g->outer_child = NULL;
  }

  return (struct generator *) g;
}
