/*  Copyright (C) 2008 Joshua Judson Rosen <rozzin@geekspace.com>.

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    version 3 as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING. If not, see
    <http://www.gnu.org/licenses/> or write to:

        The Free Software Foundation, inc.
        51 Franklin Street, Fifth Floor
        Boston, MA 02110-1301
        USA
*/

#include <stdlib.h>
#include "constants.h"
#include "random.h"

#include <math.h>
#include <cairo.h>

#include "graphics.h"
#include "symmetry.h"
#include "shape.h"

#include <glib.h>

static void
exec_symmetry (struct symmetry_generator *symmetry, cairo_t *cr)
{
  double x, y;
  int i;

  cairo_get_current_point (cr, &x, &y);
  cairo_new_sub_path (cr);

  cairo_move_to (cr, 0, 0);

  cairo_save (cr);

  for (i = 0; i < symmetry->n; i++) {
    cairo_save (cr);

    cairo_rotate (cr, 2*M_PI / symmetry->n * (i + .5));
    cairo_translate (cr, 0, symmetry->offset);
    cairo_scale (cr, symmetry->cscale, symmetry->cscale);
    if (i % 2) {
      cairo_scale (cr, -1, 1);
    }
    symmetry->child->produce (symmetry->child, cr);

    cairo_restore (cr);
  }

  cairo_restore (cr);

  cairo_new_sub_path (cr);
  cairo_move_to (cr, x, y);
}

struct generator *
gen_symmetry (int level)
{
  struct symmetry_generator *g = g_new (struct symmetry_generator, 1);

  ((struct generator *) g)->produce = (producer_fn *) exec_symmetry;

  /* XXX: this is described somewhat ambiguously in the essay: */
  g->n = pow (2, round (rndin_lowbias (1, 3, level + 2)));
  /* ^note that level starts at 0, which would mean that top-level
     symmetry-generators always use n=8; using level+1 would start
     counting at 1, which doesn't actually impose have a low bias; so,
     start counting at 2. */

  g->child = generate_child (level + 1);
  if (rndprob (0.3)) {
    g->offset = rndin (0.1, 0.5);
  } else {
    g->offset = 0;
  }
  g->cscale = rndin (0.3, 0.6) * (1 - g->offset);

  return (struct generator *) g;
}
